@@ -1,11 +1,10 @@
-
-# List to array
-# Reduce/simplify a list of homogenous objects to an array
-# 
-# @arguments list of input data
-# @arguments a data frame of labels, one row for each element of res
-# @arguments should extra dimensions be dropped (TRUE) or preserved (FALSE)
-# @keyword internal
+#' List to array.
+#' Reduce/simplify a list of homogenous objects to an array
+#' 
+#' @param res list of input data
+#' @param labels a data frame of labels, one row for each element of res
+#' @param .drop should extra dimensions be dropped (TRUE) or preserved (FALSE)
+#' @keywords internal
 list_to_array <- function(res, labels = NULL, .drop = FALSE) {
   if (length(res) == 0) return(vector())
   n <- length(res)
@@ -18,14 +17,17 @@ list_to_array <- function(res, labels = NULL, .drop = FALSE) {
       stop("Results must have the same number of dimensions.")
 
     dims <- unique(do.call("rbind", llply(res, amv_dim)))
+
+    if (is.null(dims) || !all(dims > 0))
+      stop("Results must have one or more dimensions.", call. = FALSE)
     if (nrow(dims) != 1) 
-      stop("Results must have the same dimensions.")    
+      stop("Results must have the same dimensions.", call. = FALSE)
 
     res_dim <- amv_dim(res[[1]])
     res_labels <- amv_dimnames(res[[1]])
     res_index <- expand.grid(res_labels)
 
-    res <- unlist(res)
+    res <- unname(unlist(res))
   } else {
     # Lists are degenerate case where every element is a singleton
     res_index <- as.data.frame(matrix(0, 1, 0))
@@ -47,16 +49,18 @@ list_to_array <- function(res, labels = NULL, .drop = FALSE) {
     in_dim <- sapply(in_labels, length)        
   }
   
-  index <- cbind(
+  # Generate data frame that describe result indices as they are currently
+  index <- unrowname(cbind(
     labels[rep(seq_len(nrow(labels)), each = nrow(res_index)), ,drop = FALSE],
     res_index[rep(seq_len(nrow(res_index)), nrow(labels)), , drop = FALSE]
-  )
-  
+  ))
+  # Need to be order in the opposite direction in the result
+  overall <- id(rev(index))
+
   out_dim <- unname(c(in_dim, res_dim))
   out_labels <- c(in_labels, res_labels)
   n <- prod(out_dim)
 
-  overall <- ninteraction(index)
   if (length(overall) < n) {
     overall <- match(1:n, overall, nomatch = NA)
   } else {
@@ -66,5 +70,5 @@ list_to_array <- function(res, labels = NULL, .drop = FALSE) {
   out_array <- res[overall]  
   dim(out_array) <- out_dim
   dimnames(out_array) <- out_labels
-  if (.drop) reduce(out_array) else out_array
+  if (.drop) reduce_dim(out_array) else out_array
 }
\ No newline at end of file
